<?php
/**
 * Category link modification
 *
 * @package DiagnoSEO
 */

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

/* hooks */
register_activation_hook( __FILE__, 'diagnoseo_refresh_rewrite_rules' );
register_deactivation_hook( __FILE__, 'diagnoseo_category_links_cleanup' );

$remove_category_base = get_option( 'diagnoseo_fix_category_url_base', false );
if ( $remove_category_base ) {
	/* actions */
	add_action( 'created_category', 'diagnoseo_refresh_rewrite_rules' );
	add_action( 'delete_category', 'diagnoseo_refresh_rewrite_rules' );
	add_action( 'edited_category', 'diagnoseo_refresh_rewrite_rules' );
	add_action( 'init', 'diagnoseo_permastruct' );

	/* filters */
	add_filter( 'category_rewrite_rules', 'diagnoseo_rewrite_rules' );
	add_filter( 'query_vars', 'diagnoseo_query_vars' );    // Adds 'category_redirect' query variable.
	add_filter( 'request', 'diagnoseo_request' );       // Redirects if 'category_redirect' is set.
} else {
	diagnoseo_category_links_cleanup();
}

/**
 * Flushes rewrite rules.
 */
function diagnoseo_refresh_rewrite_rules() {
	update_option( 'diagnoseo_flush_rewrite_rules', 1 );
}

/**
 * Cleans up on plugin deactivation.
 */
function diagnoseo_category_links_cleanup() {
	remove_filter( 'category_rewrite_rules', 'diagnoseo_rewrite_rules' );
	diagnoseo_refresh_rewrite_rules();
}
/**
 * Removes category base.
 *
 * @return void
 */
function diagnoseo_permastruct() {
	global $wp_rewrite, $wp_version;

	if ( 3.4 <= $wp_version ) {
		$wp_rewrite->extra_permastructs['category']['struct'] = '%category%';
	} else {
		$wp_rewrite->extra_permastructs['category'][0] = '%category%';
	}
}

/**
 * Adds category rewrite rules.
 *
 * @param Array $category_rewrite Category rewrite rules.
 */
function diagnoseo_rewrite_rules( $category_rewrite ) {
	global $wp_rewrite;

	$category_rewrite = array();

	/* WPML is present: temporary disable terms_clauses filter to get all categories for rewrite */
	if ( class_exists( 'Sitepress' ) ) {
		global $sitepress;

		remove_filter( 'terms_clauses', array( $sitepress, 'terms_clauses' ), 10 );
		$categories = get_categories(
			array(
				'hide_empty'          => false,
				'_icl_show_all_langs' => true,
			)
		);
		add_filter( 'terms_clauses', array( $sitepress, 'terms_clauses' ), 10, 4 );
	} else {
		$categories = get_categories( array( 'hide_empty' => false ) );
	}

	foreach ( $categories as $category ) {
		$category_nicename = $category->slug;
		if ( $category->parent === $category->cat_ID ) {
			$category->parent = 0;
		} elseif ( 0 !== $category->parent ) {
			$category_nicename = get_category_parents( $category->parent, false, '/', true ) . $category_nicename;
		}
		$category_rewrite[ '(' . $category_nicename . ')/(?:feed/)?(feed|rdf|rss|rss2|atom)/?$' ] = 'index.php?category_name=$matches[1]&feed=$matches[2]';
		$category_rewrite[ '(' . $category_nicename . ')/page/?([0-9]{1,})/?$' ]                  = 'index.php?category_name=$matches[1]&paged=$matches[2]';
		$category_rewrite[ '(' . $category_nicename . ')/?$' ]                                    = 'index.php?category_name=$matches[1]';
	}

	$old_category_base                                 = get_option( 'category_base', 'category' );
	$old_category_base                                 = trim( $old_category_base, '/' );
	$category_rewrite[ $old_category_base . '/(.*)$' ] = 'index.php?category_redirect=$matches[1]';

	return $category_rewrite;
}

/**
 * Adds a query variable so we know when to redirect
 *
 * @param Array $public_query_vars The query vars array.
 */
function diagnoseo_query_vars( $public_query_vars ) {
	$public_query_vars[] = 'category_redirect';

	return $public_query_vars;
}

/**
 * Handles category redirects.
 *
 * @param Array $query_vars Current query vars.
 */
function diagnoseo_request( $query_vars ) {
	if ( isset( $query_vars['category_redirect'] ) ) {
		$catlink = trailingslashit( get_option( 'home' ) ) . user_trailingslashit( $query_vars['category_redirect'], 'category' );
		status_header( 301 );
		header( "Location: $catlink" );
		exit;
	}

	return $query_vars;
}

if ( get_option( 'diagnoseo_flush_rewrite_rules' ) ) {
	delete_option( 'diagnoseo_flush_rewrite_rules' );
	flush_rewrite_rules();
}

/**
 * Product category
 */

$remove_product_cat_from_links = get_option( 'diagnoseo_fix_product_cat_url_base' );
if ( ! empty( $remove_product_cat_from_links ) ) {
	add_filter( 'request', 'diagnoseo_change_product_cat_request', 1, 1 );
	add_filter( 'term_link', 'diagnoseo_product_cat_permalink', 10, 3 );
}

/**
 * Modifies the request for the product category
 *
 * @param WP_Query $query WP_Query object.
 */
function diagnoseo_change_product_cat_request( $query ) {

	$tax_name = 'product_cat';

	// Request for child terms differs, we should make an additional check.
	if ( isset( $query['attachment'] ) && ! empty( $query['attachment'] ) ) {
		$include_children = true;
		$name             = empty( $query['attachment'] ) ? '' : $query['attachment'];
	} else {
		$include_children = false;
		$name             = empty( $query['name'] ) ? '' : $query['name'];
	}

	$term = get_term_by( 'slug', $name, $tax_name ); // get the current term to make sure it exists.

	if ( ! empty( $name ) && $term && ! is_wp_error( $term ) ) { // check it here.

		if ( $include_children ) {
			unset( $query['attachment'] );
			$parent = $term->parent;
			while ( $parent ) {
				$parent_term = get_term( $parent, $tax_name );
				$name        = $parent_term->slug . '/' . $name;
				$parent      = $parent_term->parent;
			}
		} else {
			unset( $query['name'] );
		}

		if ( 'product_cat' === $tax_name ) {
			$query[ $tax_name ] = $name;
		}
	}

	return $query;
}

/**
 * Creates product category permalink
 *
 * @param String $url The permalink.
 * @param String $term Ignored.
 * @param String $taxonomy Taxonomy name.
 */
function diagnoseo_product_cat_permalink( $url, $term, $taxonomy ) {

	if ( empty( $url ) ) {
		return '';
	}

	$taxonomy_name = 'product_cat'; // your taxonomy name here.
	$taxonomy_slug = 'product-category'; // the taxonomy slug can be different with the taxonomy name (like 'post_tag' and 'tag' ).

	if ( false === strpos( $url, $taxonomy_slug ) || $taxonomy !== $taxonomy_name ) {
		return $url;
	}

	$url = str_replace( '/' . $taxonomy_slug, '', $url );
	return $url;
}
