<?php
/**
 * Breadcrumbs
 *
 * @package DiagnoSEO
 */

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

/**
 * The breadcrumbs object
 */
class Diagnoseo_Breadcrumbs {
	/**
	 * Breadcrumb position
	 *
	 * @var $position
	 */
	private $position = 0;
	/**
	 * Whether to show home page in the breadcrumbs path
	 *
	 * @var $show_home
	 */
	private $show_home = true;
	/**
	 * Whether to show page title in the breadcrumbs path
	 *
	 * @var $show_title
	 */
	private $show_title = true;
	/**
	 * Whether to include Schema:Breadcrumbs in markup
	 *
	 * @var $schema
	 */
	private $schema;
	/**
	 * List of supported active plugins
	 *
	 * @var $active_plugins
	 */
	private $active_plugins;

	/**
	 * Prepares the breadcrumbs object, collects the necessary information
	 */
	public function __construct() {

		$this->show_home  = get_theme_mod( 'diagnoseo_show_breadcrumbs_home', DIAGNOSEO_SHOW_BREADCRUMBS_HOME );
		$this->show_title = get_theme_mod( 'diagnoseo_breadcrumbs_title', DIAGNOSEO_BREADCRUMBS_TITLE );

		$this->add_actions();
	}

	/**
	 * Checks which of the supported plugins are active
	 */
	private function check_active_plugins() {
		$this->active_plugins = array(
			'woocommerce'   => class_exists( 'Woocommerce' ),
			'tribe_events'  => class_exists( 'Tribe__Events__Main' ),
			'bbpress'       => function_exists( 'is_bbpress' ),
			'multi_builder' => class_exists( 'MultiBuilder' ),
		);
	}

	/**
	 * Adds breadcrumb functions to hooks
	 */
	private function add_actions() {
		if ( get_option( 'diagnoseo_schema_breadcrumblist', true ) ) {
			add_action( 'wp_head', array( $this, 'render_schema' ), 10 );
		}
	}

	/**
	 * Prepares the structured data for a breadcrumb list item
	 *
	 * @param String $name Breadcrumb item name.
	 * @param String $url Breadcrumb item link url.
	 */
	private function prepare_item_data( $name, $url ) {
		$this->position ++;
		$type           = '@type';
		$item           = new stdClass();
		$item->$type    = 'ListItem';
		$item->name     = $name;
		$item->item     = $url;
		$item->position = $this->position;

		$this->schema->itemListElement[] = $item;
		return $item;
	}

	/**
	 * Prepares item's ancestor page data
	 *
	 * @param Integer $post_id ID of the current post.
	 */
	private function prepare_page_ancestor_data( $post_id ) {
		$post_array = get_post_ancestors( $post_id );
		krsort( $post_array );

		foreach ( $post_array as $postid ) {
			$post = get_post( $postid );
			$this->prepare_item_data( $post->post_title, get_permalink( $post->ID ) );
		}
	}

	/**
	 * Prepares category path data for specified post
	 *
	 * @param Integer $post_id ID of the current post.
	 * @param String  $taxonomy Current taxonomy name.
	 */
	private function prepare_post_category_path_data( $post_id, $taxonomy = 'category' ) {
		$terms = wp_get_post_terms(
			$post_id,
			$taxonomy,
			array(
				'orderby' => 'parent',
				'order'   => 'DESC',
			)
		);
		if ( empty( $terms ) ) {
			return;
		}

		$ancestors = get_ancestors( $main_term->term_id, $taxonomy );
		$ancestors = array_reverse( $ancestors );
		foreach ( $ancestors as $ancestor ) {
			$ancestor = get_term( $ancestor, $taxonomy );
			if ( ! is_wp_error( $ancestor ) && ! empty( $ancestor ) ) {
				$this->prepare_item_data( $ancestor->name, get_term_link( $ancestor->slug, $taxonomy ) );
			}
		}
	}

	/**
	 * Select the term that is most specific - deepest in term tree
	 *
	 * @param Integer $post_id Id of post in question.
	 * @param String  $taxonomy Taxonomy name.
	 */
	private function select_main_term( $post_id, $taxonomy = 'category' ) {
		$terms = wp_get_post_terms(
			$post_id,
			$taxonomy,
			array(
				'orderby' => 'parent',
				'order'   => 'DESC',
			)
		);
		if ( empty( $terms ) || is_wp_error( $terms ) ) {
			return;
		}

		$ancestors = array();
		$main_term = $terms[0];
		foreach ( $terms as $term ) {
			$term_ancestors = get_ancestors( $term->term_id, $taxonomy );
			if ( count( $term_ancestors ) > count( $ancestors ) ) {
				$ancestors = $term_ancestors;
				$main_term = $term;
			}
		}
		return $main_term;
	}

	/**
	 * Prepares term path data for term
	 *
	 * @param Integer $term_id ID of the current post.
	 * @param String  $taxonomy Current taxonomy name.
	 */
	private function prepare_term_path_data( $term_id, $taxonomy = 'category' ) {
		$ancestors = array_reverse( get_ancestors( $term_id, $taxonomy ) );
		if ( empty( $ancestors ) ) {
			return;
		}
		foreach ( $ancestors as $ancestor ) {
			$ancestor_data = get_term( $ancestor, $taxonomy );
			if ( ! is_wp_error( $ancestor_data ) && ! empty( $ancestor_data ) ) {
				$this->prepare_item_data( $ancestor_data->name, get_term_link( $ancestor_data->slug, $taxonomy ) );
			}
		}
	}

	/**
	 * Prepares portfolio page item data
	 */
	private function prepare_portfolio_data() {
		$portfolio_id = get_theme_mod( 'multi_portfolio_page', 'project-archive' );
		if ( 'project-archive' === $portfolio_id ) {
			$title = __( 'Portfolio', 'diagnoseo' );
			$link  = get_post_type_archive_link( 'project' );
		} else {
			$title = get_the_title( $portfolio_id );
			$link  = get_the_permalink( $portfolio_id );
		}
		$this->prepare_page_ancestor_data( $portfolio_id );
		$this->prepare_item_data( $title, $link );
	}

	/**
	 * Prepares shop page item data
	 */
	private function prepare_shop_data() {
		if ( ! $this->active_plugins['woocommerce'] ) {
			return;
		}
		$front_page_id = absint( get_option( 'page_on_front' ) );
		$shop_page_id  = absint( wc_get_page_id( 'shop' ) );
		$shop_page     = get_post( $shop_page_id );

		if ( $front_page_id !== $shop_page_id ) {
			$this->prepare_item_data( $shop_page->post_title, get_permalink( $shop_page ) );
		}
	}

	/**
	 * Prepares page for posts data
	 */
	private function prepare_posts_page_data() {
		$posts_page = get_option( 'page_for_posts' );
		if ( get_option( 'show_on_front' ) === 'page' && ! empty( $posts_page ) ) {
			$this->prepare_page_ancestor_data( $posts_page );
			$page = get_post( $posts_page );
			$this->prepare_item_data( $page->post_title, get_the_permalink( $posts_page ) );
		}
	}

	/**
	 * Returns current URL
	 */
	private function get_current_url() {
		if ( empty( $_SERVER['HTTP_HOST'] ) ) {
			return;
		}
		if ( ! empty( $_SERVER['REQUEST_URI'] ) ) {
			$url = ( isset( $_SERVER['HTTPS'] ) && 'on' === $_SERVER['HTTPS'] ? 'https' : 'http' ) . '://' . sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) . sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) );
		} else {
			$url = ( isset( $_SERVER['HTTPS'] ) && 'on' === $_SERVER['HTTPS'] ? 'https' : 'http' ) . '://' . sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) );
		}
		return $url;
	}

	/**
	 * Checks if current page needs breadcrumbs
	 *
	 * @param Integer $post_id ID of the current post.
	 */
	private function breadcrumbs_needed( $post_id ) {
		$show_breadcrumbs = empty( $post_id ) ? '' : get_post_meta( $post_id, 'diagnoseo_show_breadcrumbs', true );
		$show_breadcrumbs = empty( $show_breadcrumbs ) ? get_theme_mod( 'diagnoseo_show_breadcrumbs', DIAGNOSEO_SHOW_BREADCRUMBS ) : $show_breadcrumbs;
		return ! ! $show_breadcrumbs;
	}

	/**
	 * Builds the breadcrumb structured data (main breadcrumbs logic)
	 */
	public function build_structured_data() {
		global $post;

		/* when not to show breadcrumbs */
		if (
			( is_front_page() && ! $this->show_home ) ||
			( ! is_search() && ! is_home() && ! $this->breadcrumbs_needed( get_the_ID() ) ) ||
			( is_home() && ! $this->breadcrumbs_needed( get_option( 'page_for_posts' ) ) )
		) {
			return new stdClass();
		}

		$this->check_active_plugins();

		/* Building breadcrumbs data */

		$context = '@context';
		$type    = '@type';

		$this->schema                  = new stdClass();
		$this->schema->$context        = 'https://schema.org';
		$this->schema->$type           = 'BreadcrumbList';
		$this->schema->itemListElement = array();

		$home_link = get_theme_mod( 'diagnoseo_breadcrumbs_home', DIAGNOSEO_BREADCRUMBS_HOME );
		$home_link = empty( $home_link ) ? get_site_url() : $home_link;
		$home_name = get_theme_mod( 'diagnoseo_breadcrumbs_home_label', DIAGNOSEO_BREADCRUMBS_HOME_LABEL );
		if ( empty( $home_name ) ) {
			$home_name = __( 'Home', 'diagnoseo' );
		}

		$this->prepare_item_data( $home_name, $home_link );

		if ( is_front_page() ) {
			return $this->schema;

		} elseif ( is_home() ) {
			$id = get_option( 'page_for_posts' );
			$this->prepare_page_ancestor_data( $id );
			$this->prepare_item_data( get_the_title( $id ), get_the_permalink( $id ) );

		} elseif ( is_attachment() ) {
			$this->prepare_post_category_path_data( $post->ID );
			$this->prepare_item_data( get_the_title( $post->post_parent ), get_permalink( $post->post_parent ) );
			$this->prepare_item_data( get_the_title(), get_permalink() );

		} elseif ( is_404() ) {
			$this->prepare_item_data( __( 'Error 404 - Not Found.', 'diagnoseo' ), '' );

		} elseif ( is_category() ) {
			$this->prepare_posts_page_data();
			$category = get_queried_object();
			$this->prepare_term_path_data( $category->term_id, 'category' );
			$this->prepare_item_data( $category->name, get_category_link( $category ) );

		} elseif ( is_tag() ) {
			$this->prepare_posts_page_data();
			$tag   = get_queried_object();
			$title = __( 'Posts tagged', 'diagnoseo' ) . ': ' . single_tag_title( '', false );
			$this->prepare_item_data( $title, get_tag_link( $tag ) );

		} elseif ( is_day() ) {
			$d = get_the_time( 'd' );
			$m = get_the_time( 'F' );
			$y = get_the_time( 'Y' );
			$this->prepare_item_data( $y, get_year_link( $y ) );
			$this->prepare_item_data( $m, get_month_link( $y, $m ) );
			$this->prepare_item_data( $d, get_day_link( $y, $m, $d ) );

		} elseif ( is_month() ) {
			$m = get_the_time( 'F' );
			$y = get_the_time( 'Y' );
			$this->prepare_item_data( $y, get_year_link( $y ) );
			$this->prepare_item_data( $m, get_month_link( $y, $m ) );

		} elseif ( is_year() ) {
			$y = get_the_time( 'Y' );
			$this->prepare_item_data( $y, get_year_link( $y ) );

		} elseif ( is_author() ) {
			$this->prepare_posts_page_data();
			global $author;
			$user  = get_userdata( $author );
			$title = __( 'Articles by', 'diagnoseo' ) . ': ' . $user->display_name;
			$url   = get_author_posts_url( $user->user_url );
			$this->prepare_item_data( $title, $url );

		} elseif ( is_search() ) {
			$title = __( 'Search results for', 'diagnoseo' ) . ': "' . get_search_query() . '"';
			$this->prepare_item_data( $title, get_search_link() );

		} elseif ( is_page() ) {
			$id = get_the_ID();
			$this->prepare_page_ancestor_data( $id );
			$page = get_post( $id );
			$this->prepare_item_data( $page->post_title, get_the_permalink( $id ) );

		} elseif ( $this->active_plugins['tribe_events'] && tribe_is_month() && ! is_tax() ) {
			$this->prepare_item_data( __( 'Events (by month)', 'diagnoseo' ), $this->get_current_url() );

		} elseif ( $this->active_plugins['tribe_events'] && tribe_is_month() && is_tax() ) {
			$this->prepare_item_data( __( 'Events', 'diagnoseo' ), tribe_get_events_link() );
			$this->prepare_item_data( tribe_get_month_view_date(), $this->get_current_url() );

		} elseif ( $this->active_plugins['tribe_events'] && ( tribe_is_past() || tribe_is_upcoming() && ! is_tax() ) ) {
			$this->prepare_item_data( __( 'Events', 'diagnoseo' ), $this->get_current_url() );

		} elseif ( $this->active_plugins['tribe_events'] && ( tribe_is_past() || tribe_is_upcoming() && is_tax() ) ) {
			$category = get_term_by( 'slug', get_query_var( 'term' ), 'tribe_events_cat' );
			$this->prepare_item_data( __( 'Events', 'diagnoseo' ), tribe_get_events_link() );
			$this->prepare_item_data( $category->name, $this->get_current_url() );

		} elseif ( $this->active_plugins['tribe_events'] && tribe_is_day() && ! is_tax() ) {
			$this->prepare_item_data( __( 'Events (by day)', 'diagnoseo' ), $this->get_current_url() );

		} elseif ( $this->active_plugins['tribe_events'] && tribe_is_day() && is_tax() ) {
			$this->prepare_item_data( __( 'Events', 'diagnoseo' ), tribe_get_events_link() );
			$this->prepare_item_data( __( 'Day view', 'diagnoseo' ), $this->get_current_url() );

		} elseif ( $this->active_plugins['tribe_events'] && tribe_is_event() && is_single() ) {
			$this->prepare_item_data( __( 'Events', 'diagnoseo' ), tribe_get_events_link() );
			$ev_cats  = get_the_terms( get_the_id(), 'tribe_events_cat' );
			$category = $ev_cats[0];
			$this->prepare_item_data( $category->name, tribe_get_listview_link( $category->term_id ) );
			$this->prepare_item_data( get_the_title(), get_the_permalink() );

		} elseif ( $this->active_plugins['tribe_events'] && tribe_is_venue() ) {
			$this->prepare_item_data( __( 'Events', 'diagnoseo' ), tribe_get_events_link() );
			$this->prepare_item_data( get_the_title(), get_the_permalink() );

		} elseif ( $this->active_plugins['tribe_events'] && 'tribe_organizer' === get_post_type() && is_single() ) {
			$this->prepare_item_data( __( 'Events', 'diagnoseo' ), tribe_get_events_link() );
			$this->prepare_item_data( get_the_title(), get_the_permalink() );

		} elseif ( is_tax( 'product_cat' ) && $this->active_plugins['woocommerce'] ) {
			$this->prepare_shop_data();
			$term = get_queried_object();
			$link = get_term_link( $term, 'product_cat' );
			$this->prepare_term_path_data( $term->term_id, 'product_cat' );
			$this->prepare_item_data( $term->name, $link );

		} elseif ( is_tax( 'product_tag' ) && $this->active_plugins['woocommerce'] ) {
			$this->prepare_shop_data();
			$term  = get_queried_object();
			$title = __( 'Products tagged &ldquo;', 'diagnoseo' ) . $term->name . '&rdquo;';
			$link  = get_term_link( $term, 'product_tag' );
			$this->prepare_item_data( $title, $link );

		} elseif ( is_tax( 'project-categories' ) && $this->active_plugins['multi_builder'] ) {
			$this->prepare_portfolio_data();

			$term = get_queried_object();
			$tax  = get_taxonomy( $term->taxonomy );

			if ( $tax->hierarchical ) {
				$this->prepare_term_path_data( $term->term_id, $tax->name );
			}
			$this->prepare_item_data( $term->name, get_term_link( $term ) );

		} elseif ( is_tax() ) {
			$term = get_queried_object();
			$tax  = get_taxonomy( $term->taxonomy );

			if ( $tax->hierarchical ) {
				$this->prepare_term_path_data( $term->term_id, $tax->name );
			}
			$this->prepare_item_data( $term->name, get_term_link( $term ) );

		} elseif ( is_single() ) {
			$type = get_post_type();
			switch ( $type ) {
				case 'project':
					$this->prepare_portfolio_data();
					$main_term = $this->select_main_term( $post->ID, 'project-categoriess' );
					if ( ! empty( $main_term ) ) {
						$this->prepare_term_path_data( $main_term->term_id, 'project-categories' );
						$this->prepare_item_data( $main_term->name, get_term_link( $main_term ) );
					}
					break;
				case 'product':
					$this->prepare_shop_data();
					$main_term = $this->select_main_term( $post->ID, 'product_cat' );
					if ( ! empty( $main_term ) ) {
						$this->prepare_term_path_data( $main_term->term_id, 'product_cat' );
						$this->prepare_item_data( $main_term->name, get_term_link( $main_term ) );
					}
					break;
				case 'post':
					$this->prepare_posts_page_data();
					$main_term = $this->select_main_term( $post->ID );
					if ( ! empty( $main_term ) ) {
						$this->prepare_term_path_data( $main_term->term_id );
						$this->prepare_item_data( $main_term->name, get_term_link( $main_term ) );
					}
					break;
				default:
					$archive_link = get_post_type_archive_link( $type );
					if ( $archive_link ) {
						$post_type     = get_post_type_object( $type );
						$archive_title = $post_type->labels->name;
						$this->prepare_item_data( $archive_title, $archive_link );
					}
					$taxonmies = get_object_taxonomies( $post );
					if ( ! empty( $taxonomies ) ) {
						foreach ( $taxonomies as $tax_name ) {
							$tax = get_taxonomy( $tax_name );
							if ( $tax->hierarchical ) {
								$main_term = $this->select_main_term( $post->ID, $tax_name );
								if ( ! empty( $main_term ) ) {
									$this->prepare_term_path_data( $main_term->term_id, $tax_name );
									$this->prepare_item_data( $main_term->name, get_term_link( $main_term ) );
								}
								break;
							}
						}
					}
			}

			$this->prepare_item_data( $post->post_title, get_the_permalink() );

		} elseif ( class_exists( 'Woocommerce' ) && ( ! is_home() && ! is_front_page() && ! ( is_post_type_archive() && get_option( 'page_on_front' ) === wc_get_page_id( 'shop' ) ) ) ) {
			$this->prepare_shop_data();

		}

		return $this->schema;
	}

	/**
	 * Returns the generated schema object
	 */
	public function get_schema() {
		return empty( $this->schema ) ? false : $this->schema;
	}

	/**
	 * Renders structured breadcrumbs data as JSON
	 */
	public function render_schema() {
		$this->build_structured_data();
		if ( empty( $this->schema ) ) {
			return;
		}
		?>
	<script type="application/ld+json">
		<?php echo wp_json_encode( $this->schema, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE ); ?>
	</script>
		<?php
	}

	/**
	 * Renders the breadcrumbs based on the structured data
	 */
	public function render_breadcrumbs() {
		if ( empty( $this->schema ) ) {
			return;
		}
		$classes  = 'diagnoseo-breadcrumbs';
		$classes .= get_theme_mod( 'diagnoseo_breadcrumbs_border', DIAGNOSEO_BREADCRUMBS_BORDER ) ? ' with-border' : '';
		if ( ! empty( get_theme_mod( 'diagnoseo_breadcrumbs_color' ) ) ) {
			$classes .= ' colored-text';
		}
		if ( ! empty( get_theme_mod( 'diagnoseo_breadcrumbs_link_color' ) ) || ! empty( get_theme_mod( 'diagnoseo_breadcrumbs_link_hover_color' ) ) ) {
			$classes .= ' colored-links';
		}
		$show_title = get_theme_mod( 'diagnoseo_breadcrumbs_title', DIAGNOSEO_BREADCRUMBS_TITLE );
		$separator  = get_theme_mod( 'diagnoseo_breadcrumbs_separator', DIAGNOSEO_BREADCRUMBS_SEPARATOR );
		$delimiter  = '<span class="delimiter" aria-hidden="true">' . esc_html( $separator ) . '</span>';
		?>
		<nav class="<?php echo esc_attr( $classes ); ?>" aria-label="Breadcrumbs">
		<?php
		if ( function_exists( 'is_bbpress' ) && is_bbpress() ) {
			echo wp_kses_post(
				bbp_get_breadcrumb(
					array(
						'before'     => '<div class="breadcrumbs-content">',
						'after'      => '</div>',
						'sep'        => esc_html( $separator ),
						'sep_before' => '<span class="delimiter" aria-hidden="true">',
						'sep_after'  => '</span>',
					)
				)
			);
			?>
			</nav>
			<?php
			return;
		}
		?>
			<ol class="breadcrumbs-content">
				<?php
				$items = $this->schema->itemListElement;
				$count = count( $items );
				foreach ( $items as $i => $item ) {
					$pos = absint( $item->position );
					if ( $pos < $count ) {
						?>
						<li>
							<?php
							if ( 1 !== $pos ) :
								?>
								<span class="delimiter" aria-hidden="true"><?php echo esc_html( $separator ); ?></span>
							<?php endif; ?>
							<a href="<?php echo esc_url( $item->item ); ?>"><?php echo esc_html( $item->name ); ?></a></li>
						<?php
					} elseif ( $show_title && $pos === $count ) {
						?>
						<li>
							<?php if ( 1 !== $count ) : ?>
							<span class="delimiter" aria-hidden="true"><?php echo esc_html( $separator ); ?></span>
							<?php endif; ?>
							<span class="current" aria-current="location"><?php echo esc_html( $item->name ); ?></span></li>
						<?php
					}
				}
				?>
			</ol>
			</nav>
		<?php
	}
}
