<?php
/**
 * IndexNow settings
 *
 * @package DiagnoSEO
 */

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

/**
 * IndexNow settings page
 */
class Diagnoseo_IndexNowSettings {

	/**
	 * Hooks actions on object creation
	 */
	public function __construct() {
		global $pagenow;
		if ( 'admin.php' === $pagenow || 'options.php' === $pagenow ) {
			add_action( 'admin_init', array( $this, 'register_settings' ) );
		}
	}

	/**
	 * Renders post meta import page
	 */
	public function render() {
		$apikey = get_option( 'diagnoseo_indexnow_apikey' );
		?>
		<div class="wrap">
			<h1><?php esc_html_e( 'IndexNow', 'diagnoseo' ); ?></h1>

			<h2><?php esc_html_e( 'Add the IndexNow API key', 'diagnoseo' ); ?></h2>
			<form method="post" action="options.php" class="diagnoseo-settings-form">
				<?php
				settings_fields( 'diagnoseo_indexnow_options' );
				do_settings_sections( 'diagnoseo_indexnow_options' );
				?>
				<table class="form-table" role="presentation">
					<tbody>
						<tr>
							<th><label for="diagnoseo_indexnow_apikey"><?php esc_html_e( 'IndexNow API key', 'diagnoseo' ); ?></label></th>
							<td>
								<input type="text" id="diagnoseo_indexnow_apikey" name="diagnoseo_indexnow_apikey" class="regular-text" value="<?php echo esc_attr( $apikey ); ?>" />
								<p class="description"><?php esc_html_e( 'Here you can paste the API key generated here:', 'diagnoseo' ); ?> <a href="https://www.bing.com/indexnow" target="_blank">https://www.bing.com/indexnow</a></p>
							</td>
						</tr>
					</tbody>
				</table>
				<?php submit_button(); ?>
			</form>

			<h2><?php esc_html_e( 'Create a file hosting your API key', 'diagnoseo' ); ?></h2>
			<form method="post" class="diagnoseo-settings-form">
				<?php wp_nonce_field( 'index_now_api_key_file', 'index_now_api_key_file_nonce' ); ?>

				<?php
				if ( isset( $_POST['index_now_api_key_file_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['index_now_api_key_file_nonce'] ) ), 'index_now_api_key_file' ) ) {
					if ( isset( $_POST['diagnoseo_indexnow_apikey_file'] ) ) {
						$this->write_apikey_file( sanitize_text_field( wp_unslash( $_POST['diagnoseo_indexnow_apikey_file'] ) ) );
					}
				}

				global $wp_filesystem;
				$dir  = $wp_filesystem->find_folder( '' );
				$file = trailingslashit( $dir ) . $apikey . '.txt';
				if ( file_exists( ABSPATH . $file ) ) :
					?>
					<p><?php esc_html_e( 'The file hosting your API key exists. No need to create a new one.', 'diagnoseo' ); ?></p>
				<?php endif; ?>

				<?php wp_nonce_field( 'filesystem-nonce' ); ?>
				<input type="hidden" id="diagnoseo_indexnow_apikey_file" name="diagnoseo_indexnow_apikey_file" value="<?php echo esc_attr( get_option( 'diagnoseo_indexnow_apikey' ) ); ?>" />
				<p><?php esc_html_e( 'Click the button below to create a new file with your API key. You may be asked for FTP/SSH credentials.', 'diagnoseo' ); ?></p>
				<p class="submit">
					<input type="submit" class="button button-primary" name="submit" value="<?php esc_html_e( 'Create file', 'diagnoseo' ); ?>" />
				</p>
			</form>
		</div>
		<?php
	}

	/**
	 * Registers script settings
	 */
	public function register_settings() {
		add_option( 'diagnoseo_indexnow_apikey', '' );
		register_setting(
			'diagnoseo_indexnow_options',
			'diagnoseo_indexnow_apikey',
			array(
				'type'        => 'string',
				'description' => __( 'API key', 'diagnoseo' ),
				'default'     => '',
			)
		);
	}

	/**
	 * Writes the API key to a file required by Bing
	 *
	 * @param String $value File name.
	 */
	public function write_apikey_file( $value ) {
		global $wp_filesystem;

		$url         = wp_nonce_url( 'admin.php?page=diagnoseo-indexnow', 'filesystem-nonce' );
		$form_fields = array( 'diagnoseo_indexnow_apikey_file' );

		if ( $this->filesystem_connect( $url, '', '/', $form_fields ) ) {
			$dir  = $wp_filesystem->find_folder( ABSPATH );
			$file = trailingslashit( $dir ) . $value . '.txt';
			$wp_filesystem->put_contents( $file, $value, FS_CHMOD_FILE );
			return $value;
		}
	}

	/**
	 * Connects to file system
	 *
	 * @param String $url The URL to post the form to.
	 * @param String $method Chosen type of filesystem.
	 * @param String $context Full path to the directory that is tested for being writable.
	 * @param Array  $fields Extra POST fields to be checked for inclusion in the post.
	 */
	private function filesystem_connect( $url, $method, $context, $fields = null ) {
		global $wp_filesystem;
		$credentials = request_filesystem_credentials( $url, $method, false, $context, $fields );
		if ( false === $credentials ) {
			return false;
		}
		if ( ! WP_Filesystem( $credentials ) ) {
			request_filesystem_credentials( $url, $method, true, $context );
			return false;
		}

		return true;
	}
}
