<?php
/**
 * Meta tag rendering
 *
 * @package DiagnoSEO
 */

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

/**
 * Renders meta tags and page title
 */
class Diagnoseo_MetaTags {
	/**
	 * Title part separator
	 *
	 * @var $separator
	 */
	private static $separator;
	/**
	 * Flag - set to true if page title has been rendered
	 *
	 * @var $title_rendered
	 */
	private static $title_rendered;

	/**
	 * Default values
	 *
	 * @var $title_rendered
	 */
	private $defaults;

	/**
	 * Constructor: adds actions to hooks
	 *
	 * @param Array $defaults Associative array of default option values.
	 */
	public function __construct( $defaults ) {
		$this->defaults = $defaults;

		self::$separator = get_option( 'diagnoseo_separator', $this->defaults['diagnoseo_separator'] );

		remove_action( 'wp_head', '_wp_render_title_tag', 1 );
		remove_action( 'wp_head', 'wp_title' );
		add_action( 'wp_head', array( $this, 'meta_comment_start' ), 1 );
		add_action( 'wp_head', array( $this, 'render_title' ), 1 );

		add_filter( 'wp_head', array( $this, 'render_meta_descr' ), 1 );
		add_filter( 'wpseo_metadesc', array( $this, 'render_meta_descr' ) );

		add_action( 'wp_head', array( $this, 'remove_meta_robots' ), 0 );
		add_filter( 'wp_robots', array( $this, 'meta_robots' ), 99 );

		if ( function_exists( 'rel_canonical' ) ) {
			remove_action( 'wp_head', 'rel_canonical' );
		}
		add_action( 'wp_head', array( $this, 'render_canonical' ) );

		add_filter( 'wp_head', array( $this, 'meta_og_tags' ), 1 );
		add_filter( 'wp_head', array( $this, 'social_meta_tags' ), 1 );
		add_action( 'wp_head', array( $this, 'meta_comment_end' ), 1 );

		add_filter( 'wp_title', array( $this, 'replace_title' ), 999 );
	}

	/**
	 * Adds comment before meta tags
	 */
	public function meta_comment_start() {
		echo "\n<!-- WordPress SEO Plugin - https://diagnoseo.com/wordpress-seo-plugin/ -->\n";
	}

	/**
	 * Adds comment after meta tags
	 */
	public function meta_comment_end() {
		echo "<!-- WordPress SEO Plugin End - https://diagnoseo.com/wordpress-seo-plugin/ -->\n\n";
	}

	/**
	 * Replaces default page title with prepared by plugin
	 */
	public function replace_title() {
		if ( self::$title_rendered ) {
			return '';
		}
		self::$title_rendered = true;
		return $this->prepare_title();
	}
	/**
	 * Outputs the title tag
	 */
	public function render_title() {
		if ( self::$title_rendered ) {
			return;
		}
		self::$title_rendered = true;
		echo '<title>' . esc_html( $this->prepare_title() ) . "</title>\n";
	}

	/**
	 * Retrieves the current page title and fills it with data
	 */
	public function prepare_title() {
		if ( is_front_page() ) {
			$home_title = get_option( 'diagnoseo_home_title', $this->defaults['diagnoseo_home_title'] );

			if ( is_singular() ) {
				global $post;
				$home_title = get_post_meta( $post->ID, 'diagnoseo_meta_title', true );
				if ( empty( $home_title ) ) {
					$home_title = $post->post_title;
				}
			}

			if ( ! empty( $home_title ) ) {
				return $this->replace_vars_title( $home_title );
			} else {
				if ( is_home() ) {
					return $this->replace_vars_title( $home_title );
				}
				$title = get_post_meta( get_the_id(), 'diagnoseo_meta_title', true );

				if ( empty( $title ) ) {
					$title = get_option( 'diagnoseo_page_title_template', $this->defaults['diagnoseo_page_title_template'] );
				}
				return $this->replace_vars_title( $title );
			}
		} else {
			if ( class_exists( 'MultiBuilder' ) && is_single() && 'project' === get_post_type() ) {
				global $post;
				$title = get_post_meta( get_the_id(), 'diagnoseo_meta_title', true );
				if ( empty( $title ) ) {
					$title = get_option( 'diagnoseo_project_title_template', $this->defaults['diagnoseo_project_title_template'] );
				}
				return $this->replace_vars_title( $title );

			} elseif ( class_exists( 'MultiBuilder' ) && is_tax( 'project-categories' ) ) {
				$template = get_option( 'diagnoseo_project_category_title_template', $this->defaults['diagnoseo_project_category_title_template'] );
				$category = get_queried_object();
				$title    = empty( $template ) ? $category->name : $template;
				return $this->replace_vars_title( $title );

			} elseif ( is_home() ) {
				global $post;
				$title = get_post_meta( get_option( 'page_for_posts' ), 'diagnoseo_meta_title', true );
				if ( empty( $title ) ) {
					$title = get_option( 'diagnoseo_page_title_template', $this->defaults['diagnoseo_page_title_template'] );
				}
				return $this->replace_vars_title( $title );

			} elseif ( is_single() ) {
				if ( isset( $title_parts ) && class_exists( 'WooCommerce' ) && 'product' === get_post_type( get_the_id() ) && function_exists( 'diagnoseo_pro_init' ) ) {
					return $title_parts;
				}
				global $post;
				$title = get_post_meta( get_the_id(), 'diagnoseo_meta_title', true );
				if ( empty( $title ) ) {
					$title = get_option( 'diagnoseo_post_title_template', $this->defaults['diagnoseo_post_title_template'] );
				}
				return $this->replace_vars_title( $title );

			} elseif ( is_page() ) {
				global $post;
				$title = get_post_meta( get_the_id(), 'diagnoseo_meta_title', true );
				if ( empty( $title ) ) {
					$title = get_option( 'diagnoseo_page_title_template', $this->defaults['diagnoseo_page_title_template'] );
				}
				return $this->replace_vars_title( $title );

			} elseif ( is_category() ) {
				$category = get_the_category();
				$template = get_option( 'diagnoseo_category_title_template', $this->defaults['diagnoseo_category_title_template'] );
				$title    = empty( $template ) ? $category[0]->cat_name : $template;
				return $this->replace_vars_title( $title );

			} elseif ( is_tag() ) {
				$template = get_option( 'diagnoseo_tag_title_template', $this->defaults['diagnoseo_tag_title_template'] );
				$title    = empty( $template ) ? single_tag_title( '', false ) : $template;
				return $this->replace_vars_title( $title );

			} elseif ( is_date() ) {
				$template = get_option( 'diagnoseo_date_title_template', $this->defaults['diagnoseo_date_title_template'] );
				$title    = empty( $template ) ? '%date% %sep% %sitename%' : $template;
				return $this->replace_vars_title( $title );

			} elseif ( is_archive() ) {
				$template = get_option( 'diagnoseo_archive_title_template', $this->defaults['diagnoseo_archive_title_template'] );
				$title    = empty( $template ) ? single_tag_title( '', false ) : $template;
				return $this->replace_vars_title( $title );
			}
		}
		return wp_title( self::$separator, false );
	}

	/**
	 * Replaces variables in page title with data
	 *
	 * @param String $text Text in which to replace variables.
	 */
	public function replace_vars_title( $text ) {
		if ( empty( $text ) ) {
			return '';
		}
		global $post;
		$title = '';
		$date  = '';
		if ( is_home() ) {
			$title = get_the_title( get_option( 'page_for_posts' ) );
		} elseif ( is_singular() ) {
			$title = get_the_title( $post->ID );
			$date  = get_the_date();
		} elseif ( is_category() ) {
			$title = single_cat_title( '', false );
		} elseif ( is_tag() ) {
			$title = single_tag_title( '', false );
		} elseif ( is_date() ) {
			$title = wp_strip_all_tags( get_the_archive_title() );
			if ( is_year() ) {
				$date = get_query_var( 'year' );
			} elseif ( is_month() ) {
				$monthnum = get_query_var( 'monthnum' );
				$month    = $GLOBALS['wp_locale']->get_month( $monthnum );
				$date     = $month . ' ' . get_query_var( 'year' );
			} else {
				$date = get_query_var( 'year' ) . '/' . get_query_var( 'monthnum' ) . '/' . get_query_var( 'day' );
			}
		} elseif ( is_archive() ) {
			$title = wp_strip_all_tags( get_the_archive_title() );
		}

		$vars    = array( '%title%', '%page%', '%sitename%', '%sep%', '%tagline%', '%currentyear%', '%currentmonth%', '%date%' );
		$replace = array(
			html_entity_decode( $title ),
			get_query_var( 'paged' ) ? get_query_var( 'paged' ) : '',
			get_bloginfo( 'name' ),
			self::$separator,
			get_bloginfo( 'description' ),
			gmdate( 'Y' ),
			gmdate( 'F' ),
			$date,
		);

		return str_replace( $vars, $replace, $text );
	}

	/**
	 * Renders meta description tag
	 */
	public function render_meta_descr() {
		$description = $this->prepare_meta_descr();
		if ( ! empty( $description ) ) {
			echo '<meta name="description" content="' . esc_attr( $description ) . '" />' . "\n";
		}
	}

	/**
	 * Retrieves meta description and populates it with data
	 */
	public function prepare_meta_descr() {
		$description = '';
		if ( is_front_page() ) {
			if ( is_singular() ) {
				global $post;
				$description = get_post_meta( $post->ID, 'diagnoseo_meta_description', true );
				if ( ! empty( $description ) ) {
					return $this->replace_vars_descr( $description );
				}
			}

			$description = get_option( 'diagnoseo_home_description', $this->defaults['diagnoseo_home_description'] );
			if ( empty( $description ) && is_singular() ) {
				$description = get_post_meta( get_the_id(), 'diagnoseo_meta_description', true );
				if ( empty( $description ) ) {
					$description = get_option( 'diagnoseo_page_description_template', $this->defaults['diagnoseo_page_description_template'] );
				}
			}
		} else {
			if ( class_exists( 'MultiBuilder' ) && is_single() && 'project' === get_post_type() ) {
				$description = get_post_meta( get_the_id(), 'diagnoseo_meta_description', true );
				if ( empty( $description ) ) {
					$description = get_option( 'diagnoseo_project_description_template', $this->defaults['diagnoseo_project_description_template'] );
				}
			} elseif ( class_exists( 'MultiBuilder' ) && is_tax( 'project-categories' ) ) {
				$description = get_option( 'diagnoseo_project_category_description_template' );

			} elseif ( is_home() ) {
				$description = get_post_meta( get_option( 'page_for_posts' ), 'diagnoseo_meta_description', true );
				if ( empty( $description ) ) {
					$description = get_option( 'diagnoseo_page_description_template', $this->defaults['diagnoseo_page_description_template'] );
				}
			} elseif ( is_single() ) {
				if ( class_exists( 'WooCommerce' ) && 'product' === get_post_type( get_the_id() ) && function_exists( 'diagnoseo_pro_init' ) ) {
					return;
				}
				global $post;
				$description = get_post_meta( get_the_id(), 'diagnoseo_meta_description', true );
				if ( empty( $description ) ) {
					$description = get_option( 'diagnoseo_post_description_template', $this->defaults['diagnoseo_post_description_template'] );
				}
			} elseif ( is_page() ) {
				$description = get_post_meta( get_the_id(), 'diagnoseo_meta_description', true );
				if ( empty( $description ) ) {
					$description = get_option( 'diagnoseo_page_description_template', $this->defaults['diagnoseo_page_description_template'] );
				}
			} elseif ( is_category() ) {
				$description = wp_strip_all_tags( get_option( 'diagnoseo_category_description_template', $this->defaults['diagnoseo_category_description_template'] ) );
			} elseif ( is_tag() ) {
				$description = wp_strip_all_tags( get_option( 'diagnoseo_tag_description_template', $this->defaults['diagnoseo_tag_description_template'] ) );
			} elseif ( is_archive() ) {
				$description = wp_strip_all_tags( get_option( 'diagnoseo_archive_description_template', $this->defaults['diagnoseo_archive_description_template'] ) );
			}
		}
		return $this->replace_vars_descr( $description );
	}

	/**
	 * Replaces variables in meta description with proper data
	 *
	 * @param String $text Text in which to replace variables.
	 */
	public function replace_vars_descr( $text ) {
		if ( empty( $text ) ) {
			return '';
		}

		if ( is_front_page() ) {
			$post = null;
		} elseif ( is_home() ) {
			$post = get_post( get_option( 'page_for_posts' ) );
		} else {
			global $post;
		}
		$title   = '';
		$excerpt = '';
		$date    = '';

		if ( is_front_page() ) {
			$title   = '';
			$excerpt = '';
		} elseif ( is_singular() || is_home() ) {
			$title   = $post->post_title;
			$excerpt = get_the_excerpt( $post );
		} elseif ( is_category() ) {
			$title   = single_cat_title( '', false );
			$excerpt = category_description();
		} elseif ( is_tag() ) {
			$title   = single_tag_title( '', false );
			$excerpt = '';
		} elseif ( is_date() ) {
			$title = wp_strip_all_tags( get_the_archive_title() );
			if ( is_year() ) {
				$date = get_query_var( 'year' );
			} elseif ( is_month() ) {
				$monthnum = get_query_var( 'monthnum' );
				$month    = $GLOBALS['wp_locale']->get_month( $monthnum );
				$date     = $month . ' ' . get_query_var( 'year' );
			} else {
				$date = get_query_var( 'year' ) . '/' . get_query_var( 'monthnum' ) . '/' . get_query_var( 'day' );
			}
			$excerpt = '';

		} elseif ( is_archive() ) {
			$title   = wp_strip_all_tags( get_the_archive_title() );
			$excerpt = '';
		}

		$vars    = array( '%sep%', '%title%', '%excerpt%', '%sitename%', '%tagline%', '%currentmonth%', '%currentyear%', '%date%' );
		$replace = array(
			self::$separator,
			html_entity_decode( $title ),
			wp_strip_all_tags( html_entity_decode( $excerpt ) ),
			get_bloginfo( 'name' ),
			get_bloginfo( 'description' ),
			gmdate( 'F' ),
			gmdate( 'Y' ),
			$date,
		);

		return str_replace( $vars, $replace, $text );
	}

	/**
	 * Removes meta robots tag if needed
	 */
	public function remove_meta_robots() {
		if ( is_singular() ) {
			remove_filter( 'wp_robots', 'wp_robots_noindex' );
		}
	}

	/**
	 * Fetches data and prepares the meta robots tag
	 *
	 * @param Array $robots Current Meta Robots value array.
	 */
	public function meta_robots( $robots ) {
		$is_public = get_option( 'blog_public' );

		$public_index  = $is_public ? 'index' : 'noindex';
		$public_follow = $is_public ? 'follow' : 'nofollow';

		if ( is_singular() || is_home() ) {

			if ( is_home() ) {
				$posts_page_id = get_option( 'page_for_posts' );
				$post          = get_post( $posts_page_id );
			} else {
				global $post;
			}

			if ( is_home() && empty( $posts_page_id ) ) {
				$index  = $public_index;
				$follow = $public_follow;
			} else {
				$type = get_post_type( $post );

				$index  = get_post_meta( $post->ID, 'diagnoseo_meta_robots_index', true );
				$follow = get_post_meta( $post->ID, 'diagnoseo_meta_robots_follow', true );

				if ( empty( $index ) ) {
					$index = get_option( 'diagnoseo_' . $type . '_index', 'index' );
				}
				if ( empty( $follow ) ) {
					$follow = get_option( 'diagnoseo_' . $type . '_follow', 'follow' );
				}
			}

			if ( empty( $index ) ) {
				$index = $public_index;
			}

			if ( empty( $follow ) ) {
				$index = $public_follow;
			}

			$robots['index']    = 'index' === $index;
			$robots['noindex']  = 'index' !== $index;
			$robots['follow']   = 'follow' === $follow;
			$robots['nofollow'] = 'follow' !== $follow;

			return $robots;

		} elseif ( is_author() ) {
			$temp_i = get_option( 'diagnoseo_author_index', 'index' );
			$temp_f = get_option( 'diagnoseo_author_follow', 'follow' );
			$index  = empty( $temp_i ) ? $public_index : $temp_i;
			$follow = empty( $temp_f ) ? $public_follow : $temp_f;

			$robots['index']    = 'index' === $index;
			$robots['noindex']  = 'index' !== $index;
			$robots['follow']   = 'follow' === $follow;
			$robots['nofollow'] = 'follow' !== $follow;

		} elseif ( is_tag() ) {
			$index  = $is_public ? get_option( 'diagnoseo_tag_index', 'index' ) : 'noindex';
			$follow = $is_public ? get_option( 'diagnoseo_tag_follow', 'follow' ) : 'nofollow';
			$temp_i = get_option( 'diagnoseo_tag_index', 'index' );
			$temp_f = get_option( 'diagnoseo_tag_follow', 'follow' );
			$index  = empty( $temp_i ) ? $public_index : $temp_i;
			$follow = empty( $temp_f ) ? $public_follow : $temp_f;

			$robots['index']    = 'index' === $index;
			$robots['noindex']  = 'index' !== $index;
			$robots['follow']   = 'follow' === $follow;
			$robots['nofollow'] = 'follow' !== $follow;

		} elseif ( is_date() ) {
			$temp_i = get_option( 'diagnoseo_date_archive_index', 'index' );
			$temp_f = get_option( 'diagnoseo_date_archive_follow', 'follow' );
			$index  = empty( $temp_i ) ? $public_index : $temp_i;
			$follow = empty( $temp_f ) ? $public_follow : $temp_f;

			$robots['index']    = 'index' === $index;
			$robots['noindex']  = 'index' !== $index;
			$robots['follow']   = 'follow' === $follow;
			$robots['nofollow'] = 'follow' !== $follow;

		} elseif ( is_archive() ) {
			$temp_i = get_option( 'diagnoseo_archive_index', 'index' );
			$temp_f = get_option( 'diagnoseo_archive_follow', 'follow' );
			$index  = empty( $temp_i ) ? $public_index : $temp_i;
			$follow = empty( $temp_f ) ? $public_follow : $temp_f;

			$robots['index']    = 'index' === $index;
			$robots['noindex']  = 'index' !== $index;
			$robots['follow']   = 'follow' === $follow;
			$robots['nofollow'] = 'follow' !== $follow;
		}
		return $robots;
	}

	/**
	 * Renders the canonical link tag
	 */
	public function render_canonical() {
		if ( is_singular() || is_home() ) {
			if ( is_home() ) {
				$post = get_post( get_option( 'page_for_posts' ) );
			} else {
				global $post;
			}

			$canonical = get_post_meta( $post->ID, 'diagnoseo_meta_canonical', true );
			if ( ! empty( $canonical ) ) {
				?>
				<link rel="canonical" href="<?php echo esc_url( $canonical ); ?>" />
				<?php
			} else {
				rel_canonical();
			}
		} else {
			rel_canonical();
		}
	}

	/**
	 * Adds OpenGraph meta tags
	 */
	public function meta_og_tags() {
		if ( is_singular() || is_home() ) {
			if ( is_home() ) {
				$posts_page_id = get_option( 'page_for_posts' );
				$post          = get_post( $posts_page_id );
				$og_url        = empty( $posts_page_id ) ? home_url() : get_permalink( $posts_page_id );
			} else {
				global $post;
				$og_url = get_permalink( $post->ID );
			}

			if ( is_home() && empty( $posts_page_id ) ) {
				$title       = $this->replace_vars_title( get_option( 'diagnoseo_home_title' ) );
				$description = $this->replace_vars_descr( get_option( 'diagnoseo_home_description' ) );
			} else {
				$title = get_post_meta( $post->ID, 'diagnoseo_meta_facebook_title', true );
				if ( empty( $title ) ) {
					$title = get_post_meta( $post->ID, 'diagnoseo_meta_title', true );
					$title = $this->replace_vars_title( $title );
					if ( empty( $title ) ) {
						$type     = is_home() ? 'page' : $post->post_type;
						$template = get_option( 'diagnoseo_' . $type . '_title_template' );
						$title    = $this->replace_vars_title( $template );
						if ( empty( $title ) ) {
							$title = get_the_title( $post->ID );
						}
					}
				}
				$title = $this->replace_vars_title( $title );

				$description = get_post_meta( $post->ID, 'diagnoseo_meta_facebook_description', true );
				if ( empty( $description ) ) {
					$description = $this->prepare_meta_descr();
				}
				$description = $this->replace_vars_descr( $description );
			}

			?>
<meta property="og:type" content="article" />
<meta property="og:title" content="<?php echo esc_attr( html_entity_decode( $title ) ); ?>" />
<meta property="og:description" content="<?php echo esc_attr( html_entity_decode( $description ) ); ?>" />
<meta property="og:url" content="<?php echo esc_url( $og_url ); ?>" />
<meta property="og:site_name" content="<?php echo esc_attr( html_entity_decode( get_bloginfo( 'name' ) ) ); ?>" />
<meta property="article:publisher" content="<?php echo isset( $_SERVER['SERVER_NAME'] ) ? esc_url_raw( wp_unslash( $_SERVER['SERVER_NAME'] ) ) : ''; ?>" />
<?php //phpcs:ignore
			if ( is_singular() || ( is_home() && ! empty( $posts_page_id ) ) ) {
				$image = get_the_post_thumbnail_url( $post, array( 1200, 628 ) );

				if ( empty( $image ) ) {
					$image = get_option( 'diagnoseo_facebook_image', '' );
				}

				if ( ! empty( $image ) ) :
//phpcs:ignore
?>
<meta property="og:image" content="<?php echo esc_url( $image ); ?>" />
<meta property="og:image:width" content="1200" />
<meta property="og:image:height" content="628" />
<?php //phpcs:ignore
				endif;
			}
		}
	}

	/**
	 * Renders social meta tags for facebook & twitter
	 */
	public function social_meta_tags() {
		if ( is_singular() || is_home() ) {

			if ( is_home() ) {
				$posts_page_id = get_option( 'page_for_posts' );
				$post          = get_post( $posts_page_id );
				$url           = empty( $posts_page_id ) ? home_url() : get_permalink( $posts_page_id );
			} else {
				global $post;
				$url = get_permalink( $post->ID );
			}

			if ( is_home() && is_front_page() ) {
				$title          = get_option( 'diagnoseo_home_title' );
				$facebook_title = $title;
				$twitter_title  = $title;

				$description          = get_option( 'diagnoseo_home_description' );
				$facebook_description = $description;
				$twitter_description  = $description;

			} else {
				$post_title          = get_post_meta( $post->ID, 'diagnoseo_meta_title', true );
				$post_descr          = get_post_meta( $post->ID, 'diagnoseo_meta_description', true );
				$post_facebook_title = get_post_meta( $post->ID, 'diagnoseo_meta_facebook_title', true );
				$post_facebook_descr = get_post_meta( $post->ID, 'diagnoseo_meta_facebook_description', true );
				$post_twitter_title  = get_post_meta( $post->ID, 'diagnoseo_meta_twitter_title', true );
				$post_twitter_descr  = get_post_meta( $post->ID, 'diagnoseo_meta_twitter_description', true );
				if ( is_front_page() ) {
					$default_title = get_option( 'diagnoseo_home_title' );
					$default_descr = get_option( 'diagnoseo_home_description' );
				} else {
					$default_title = get_option( 'diagnoseo_post_title_template' );
					$default_descr = get_option( 'diagnoseo_post_description_template' );
				}

				if ( empty( $default_title ) ) {
					$default_title = $post->post_title;
				}
				if ( empty( $default_descr ) ) {
					$default_descr = get_the_excerpt( $post->ID );
				}

				$facebook_title = empty( $post_facebook_title )
					? empty( $post_title ) ? $default_title : $post_title
					: $post_facebook_title;

				$facebook_description = empty( $post_facebook_descr )
					? empty( $post_descr ) ? $default_descr : $post_descr
					: $post_facebook_descr;

				$twitter_title = empty( $post_twitter_title )
					? empty( $post_title ) ? $default_title : $post_title
					: $post_twitter_title;

				$twitter_description = empty( $post_twitter_descr )
					? empty( $post_descr ) ? $default_descr : $post_descr
					: $post_twitter_descr;
			}

			$facebook_title       = $this->replace_vars_title( $facebook_title );
			$facebook_description = $this->replace_vars_descr( $facebook_description );
			$twitter_title        = $this->replace_vars_title( $twitter_title );
			$twitter_description  = $this->replace_vars_descr( $twitter_description );

			$twitter_image = get_the_post_thumbnail_url();
			if ( empty( $twitter_image ) ) {
				$twitter_image = empty( get_option( 'diagnoseo_twitter_image' ) ) ? '' : get_option( 'diagnoseo_twitter_image' );
			}

			$twitter_card = get_option( 'diagnoseo_twitter_card' );
			$card         = empty( $twitter_image ) ? 'summary' : $twitter_card;
//phpcs:ignore
?>
<meta name="twitter:card" content="<?php echo esc_attr( $card ); ?>" />
<meta name="twitter:title" content="<?php echo esc_attr( html_entity_decode( $twitter_title ) ); ?>" />
<meta name="twitter:description" content="<?php echo esc_attr( html_entity_decode( $twitter_description ) ); ?>" />
<meta name="twitter:url" content="<?php echo esc_url( $url ); ?>" />
<?php if ( ! empty( $twitter_image ) ) : //phpcs:ignore ?>
<meta name="twitter:image" content="<?php echo esc_url( $twitter_image ); ?>" />
<?php endif; ?>
<?php if ( ! empty( $facebook_title ) ) : //phpcs:ignore ?>
<meta name="facebook:title" content="<?php echo esc_attr( html_entity_decode( $facebook_title ) ); ?>" />
<?php endif; ?>
<?php if ( ! empty( $facebook_description ) ) : //phpcs:ignore?>
<meta name="facebook:description" content="<?php echo esc_attr( html_entity_decode( $facebook_description ) ); ?>" />
<?php endif; ?>
<?php //phpcs:ignore
		}
	}
}
