<?php
/**
 * Structured data generation
 *
 * @package DiagnoSEO
 */

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

/**
 * Gets post data and prints generated schema
 */
function diagnoseo_post_schema() {
	include DIAGNOSEO_INCLUDES_PATH . 'option-defaults.php';

	if ( ! is_singular() ) {
		return;
	}
	$is_attachment    = is_attachment();
	$data             = get_post_custom( get_the_ID() );
	$post_schema_type = isset( $data['diagnoseo_schema_type'] ) ? $data['diagnoseo_schema_type'][0] : '';
	$post_schema_data = isset( $data['diagnoseo_schema_data'] ) ? json_decode( $data['diagnoseo_schema_data'][0] ) : '';

	if ( ! empty( $post_schema_type ) && ! empty( $post_schema_data ) ) {
		if ( $is_attachment ) {
			$post_schema_type = 'ImageObject';
		}
		if ( empty( $post_schema_type ) ) {
			$post_type        = get_post_type();
			$post_schema_type = get_option( 'diagnoseo_' . $post_type . '_schema_type', $diagnoseo_defaults[ 'diagnoseo_' . $post_type . '_schema_type' ] );
		}

		if ( ( empty( $post_schema_type ) || empty( $post_schema_data ) ) && ! $is_attachment ) {
			return;
		}

		$schema_function = 'diagnoseo_schema_' . strtolower( $post_schema_type );
		$schema          = $schema_function( $post_schema_data );
	} elseif ( 'none' !== $post_schema_type ) {
		$post_data = get_post();

		$option_name = 'diagnoseo_' . $post_data->post_type . '_schema_type';
		$schema_type = get_option( $option_name, isset( $diagnoseo_defaults[ $option_name ] ) ? $diagnoseo_defaults[ $option_name ] : '' );

		$type    = '@type';
		$context = '@context';
		$id      = '@id';

		switch ( $post_data->post_type ) {
			case 'page':
				$allowed_schemas = array( 'WebPage', 'Article' );
				break;
			default:
				$allowed_schemas = array( 'BlogPosting', 'NewsArticle', 'Article', 'WebPage' );
		}

		if ( in_array( $schema_type, $allowed_schemas, true ) ) {
			$schema                = new stdClass();
			$schema->$context      = 'https://schema.org';
			$schema->$type         = $schema_type;
			$schema->url           = get_the_permalink( $post_data->ID );
			$schema->headline      = $post_data->post_title;
			$schema->commentCount  = get_comment_count( $post_data->ID )['approved']; // phpcs:ignore
			$schema->datePublished = get_the_date( 'c', $post_data->ID ); // phpcs:ignore
			if ( has_post_thumbnail( $post_data->ID ) ) {
				$schema->image = get_the_post_thumbnail_url( $post_data->ID );
			}

			$schema->author        = new stdClass();
			$schema->author->$type = 'Person';
			$schema->author->name  = get_the_author_meta( 'display_name', $post_data->post_author );

			$reviewedby_name = get_option( 'diagnoseo_reviewedby_name', '' );

			if ( ! empty( $reviewedby_name ) ) {
				if ( 'WebPage' === $schema_type ) {
					$schema->reviewedBy = diagnoseo_get_reviewedby_schema( null ); // phpcs:ignore
				} else {
					$main_entity             = new stdClass();
					$main_entity->$type      = 'WebPage';
					$main_entity->$id        = get_the_permalink( $post_data->ID );
					$main_entity->name       = get_the_title( $post_data->ID );
					$main_entity->reviewedBy = diagnoseo_get_reviewedby_schema( null ); // phpcs:ignore

					$schema->mainEntityOfPage = $main_entity; // phpcs:ignore
				}
			}
		}
	}

	if ( ! empty( $schema ) ) :
		?>
<!-- DiagnoSEO Post Structured Data -->
<script type="application/ld+json"><?php echo wp_json_encode( $schema, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE ); ?></script>
<!-- DiagnoSEO Post Structured Data End -->
		<?php
	endif;
}
add_action( 'wp_head', 'diagnoseo_post_schema' );

/**
 * Generates BlogPosting structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_blogposting( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;

	$schema = diagnoseo_create_schema_object( 'BlogPosting' );

	if ( ! empty( $data->headline ) ) {
		$schema->headline = $data->headline;
		$schema->name     = $data->headline;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	if ( ! empty( $data->keywords ) ) {
		$schema->keywords = $data->keywords;
	}

	$schema->datePublished = get_the_time( 'c' ); // phpcs:ignore
	$schema->dateModified  = 'xxxx' . get_the_modified_time( 'c' ); // phpcs:ignore
	$schema->inLanguage    = get_locale(); // phpcs:ignore
	$schema->author        = diagnoseo_get_author_schema(); // phpcs:ignore
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Article structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_article( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;

	$schema = diagnoseo_create_schema_object( 'Article' );

	if ( ! empty( $data->headline ) ) {
		$schema->headline = esc_attr( $data->headline );
		$schema->name     = esc_attr( $data->headline );
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = esc_attr( $data->description );
	}
	if ( ! empty( $data->keywords ) ) {
		$schema->keywords = esc_attr( $data->keywords );
	}
	$schema->datePublished = get_the_time( 'c' ); // phpcs:ignore
	$schema->dateModified  = get_the_modified_time( 'c' ); // phpcs:ignore
	$schema->inLanguage    = get_locale(); // phpcs:ignore
	$schema->author        = diagnoseo_get_author_schema(); // phpcs:ignore
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates NewsArticle structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_newsarticle( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;

	$schema = diagnoseo_create_schema_object( 'NewsArticle' );

	if ( ! empty( $data->headline ) ) {
		$schema->headline = esc_attr( $data->headline );
		$schema->name     = esc_attr( $data->headline );
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = esc_attr( $data->description );
	}
	if ( ! empty( $data->keywords ) ) {
		$schema->keywords = esc_attr( $data->keywords );
	}
	$schema->datePublished = get_the_time( 'c' ); // phpcs:ignore
	$schema->dateModified  = get_the_modified_time( 'c' ); // phpcs:ignore
	$schema->inLanguage    = get_locale(); // phpcs:ignore
	$schema->author        = diagnoseo_get_author_schema(); // phpcs:ignore
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Review structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_review( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	$type = '@type';

	$schema = diagnoseo_create_schema_object( 'Review' );

	$schema->itemReviewed = new stdClass(); // phpcs:ignore
	if ( ! empty( $data->reviewed_item ) ) {
		$schema->itemReviewed->$type = $data->reviewed_item; // phpcs:ignore
	}
	if ( ! empty( $data->reviewed_item_name ) ) {
		$schema->itemReviewed->name = $data->reviewed_item_name; // phpcs:ignore
	}
	if ( ! empty( $data->reviewed_item_image ) ) {
		$schema->itemReviewed->image = $data->reviewed_item_image; // phpcs:ignore
	}

	if ( ! empty( $data->rating ) ) {
		$schema->reviewRating              = new stdClass(); // phpcs:ignore
		$schema->reviewRating->$type       = 'Rating'; // phpcs:ignore
		$schema->reviewRating->ratingValue = $data->rating; // phpcs:ignore
	}

	if ( ! empty( $data->title ) ) {
		$schema->name = $data->title;
	}
	if ( ! empty( $data->review_body ) ) {
		$schema->reviewBody = $data->review_body; // phpcs:ignore
	}
	$schema->author = diagnoseo_get_author_schema();

	if ( ! empty( $data->publisher_name ) ) {
		$schema->publisher        = new stdClass();
		$schema->publisher->$type = $data->publisher_type;
		$schema->publisher->name  = $data->publisher_name;
	}

	return $schema;
}

/**
 * Generates Service structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_service( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';

	$schema = diagnoseo_create_schema_object( 'Service' );

	if ( ! empty( $data->service_name ) ) {
		$schema->name = $data->service_name;
	}
	if ( ! empty( $data->service_description ) ) {
		$schema->description = $data->service_description;
	}
	if ( ! empty( $data->service_type ) ) {
		$schema->serviceType = $data->service_type; // phpcs:ignore
	}
	$schema->url = get_the_permalink();

	if ( ! empty( $data->offer_price ) ) {
		$schema->offers        = new stdClass();
		$schema->offers->$type = 'Offer';
		$schema->offers->price = $data->offer_price;
		if ( ! empty( $data->offer_currency ) ) {
			$schema->offers->priceCurrency = $data->offer_currency;
		}
		$schema->offers->availability = 'InStock';
	}

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Product structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_product( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';

	$schema = diagnoseo_create_schema_object( 'Product' );

	if ( ! empty( $data->name ) ) {
		$schema->name = $data->name;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	if ( ! empty( $data->sku ) ) {
		$schema->sku = $data->sku;
	}
	if ( ! empty( $data->identifier_name ) && ! empty( $data->identifier_value ) ) {
		$identifier          = $data->identifier_name;
		$schema->$identifier = $data->identifier_value;
	}
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}
	if ( ! empty( $data->brand ) ) {
		$schema->brand        = new stdClass();
		$schema->brand->$type = 'Brand';
		$schema->brand->name  = $data->brand;
	}

	if ( ! empty( $data->price ) ) {
		$schema->offers        = new stdClass();
		$schema->offers->$type = 'Offer';
		$schema->offers->url   = get_the_permalink();
		$schema->offers->price = $data->price;
		if ( ! empty( $data->currency ) ) {
			$schema->offers->priceCurrency = $data->currency;
		}
		$schema->offers->availability = $data->availability;
		if ( ! empty( $data->deadline ) ) {
			$schema->offers->priceValidUntil = $data->deadline;
		}
	}

	$schema->review = diagnoseo_get_rating_schema( $data );

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Course structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_course( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';

	$schema = diagnoseo_create_schema_object( 'Course' );

	if ( ! empty( $data->name ) ) {
		$schema->name = $data->name;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}

	if ( ! empty( $data->provider_name ) ) {
		$schema->provider        = new stdClass();
		$schema->provider->$type = $data->provider_type;
		$schema->provider->name  = $data->provider_name;
		if ( ! empty( $data->provider_url ) ) {
			$schema->provider->sameAs = $data->provider_url;
		}
	}

	$schema->review = diagnoseo_get_rating_schema( $data );

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Recipe structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_recipe( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';
	$id   = '@id';

	$schema = diagnoseo_create_schema_object( 'Recipe' );

	if ( ! empty( $data->name ) ) {
		$schema->name = $data->name;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	$schema->datePublished = get_the_time( 'c' ); // phpcs:ignore
	$schema->inLanguage    = get_locale(); // phpcs:ignore
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}

	if ( ! empty( $data->prep_time ) ) {
		$schema->prepTime = $data->prep_time; // phpcs:ignore
	}
	if ( ! empty( $data->cook_time ) ) {
		$schema->cookTime = $data->cook_time; // phpcs:ignore
	}
	if ( ! empty( $data->total_time ) ) {
		$schema->totalTime = $data->total_time; // phpcs:ignore
	}
	if ( ! empty( $data->type ) ) {
		$schema->recipeCategory = $data->type; // phpcs:ignore
	}
	if ( ! empty( $data->cuisine ) ) {
		$schema->recipeCuisine = $data->cuisine; // phpcs:ignore
	}
	if ( ! empty( $data->keywords ) ) {
		$schema->keywords = $data->keywords;
	}
	if ( ! empty( $data->yield ) ) {
		$schema->recipeYield = $data->yield; // phpcs:ignore
	}
	if ( ! empty( $data->ingredients ) ) {
		$schema->recipeIngredient = $data->ingredients; // phpcs:ignore
	}
	if ( ! empty( $data->instructions ) ) {
		$schema->recipeInstructions = $data->instructions; // phpcs:ignore
	}

	if ( ! empty( absint( $data->calories ) ) ) {
		$schema->nutrition           = new stdClass();
		$schema->nutrition->$type    = 'NutritionInformation';
		$schema->nutrition->calories = $data->calories;
	}

	$schema->author        = new stdClass();
	$schema->author->$type = 'Person';
	$author_info           = get_user_by( 'ID', $post->post_author );
	$schema->author->url   = get_author_posts_url( $post->post_author );
	$schema->author->name  = $author_info->display_name;

	if ( ! empty( $data->video_url ) ) {
		$schema->video           = new stdClass();
		$schema->video->$type    = 'VideoObject';
		$schema->video->embedUrl = $data->video_url;

		if ( ! empty( $data->video_title ) ) {
			$schema->video->name = $data->video_title;
		}
		if ( ! empty( $data->video_description ) ) {
			$schema->video->description = $data->video_description;
		}
		if ( ! empty( $data->video_content_url ) ) {
			$schema->video->contentUrl = $data->video_content_url;
		}
		if ( ! empty( $data->video_thumbnail ) ) {
			$schema->video->thumbnailUrl = $data->video_thumbnail;
		}
		if ( ! empty( $data->video_duration ) ) {
			$schema->video->duration = $data->video_duration;
		}
		if ( ! empty( $data->video_uploaded ) ) {
			$schema->video->uploadDate = $data->video_uploaded;
		}
	}

	$schema->aggregateRating = diagnoseo_get_aggregate_rating_schema( $data ); // phpcs:ignore

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Person structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_person( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';
	$id   = '@id';

	$schema = diagnoseo_create_schema_object( 'Person' );

	if ( ! empty( $data->name ) ) {
		$schema->name = $data->name;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	if ( ! empty( $data->email ) ) {
		$schema->email = $data->email;
	}
	if ( ! empty( $data->url ) ) {
		$schema->url = $data->url;
	}
	if ( ! empty( $data->gender ) ) {
		$schema->gender = $data->gender;
	}
	if ( ! empty( $data->job_title ) ) {
		$schema->jobTitle = $data->job_title; // phpcs:ignore
	}
	if ( ! empty( $data->image ) ) {
		$schema->image      = new stdClass();
		$schema->image->$id = esc_url( $data->image );
	}

	$schema->address = diagnoseo_get_address_schema( $data );

	return $schema;
}

/**
 * Generates JobPosting structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_jobposting( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';
	$id   = '@id';

	$schema = diagnoseo_create_schema_object( 'JobPosting' );

	if ( ! empty( $data->title ) ) {
		$schema->title = $data->title;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	$schema->datePosted = empty( $data->date_posted ) ? get_the_time( 'c' ) : $data->date_posted; // phpcs:ignore
	if ( ! empty( $data->date_expires ) ) {
		$schema->validThrough = $data->date_expires; // phpcs:ignore
	}
	$schema->employmentType = array( $data->employment_type ); // phpcs:ignore
	if ( ! empty( $data->posting_id ) ) {
		$schema->id = $data->posting_id;
	}

	if ( ! empty( $data->salary_amount ) ) {
		$schema->baseSalary        = new stdClass(); // phpcs:ignore
		$schema->baseSalary->$type = 'MonetaryAmount'; // phpcs:ignore
		if ( ! empty( $data->salary_currency ) ) {
			$schema->baseSalary->currency = $data->salary_currency; // phpcs:ignore
		}
		$schema->baseSalary->value        = new stdClass(); // phpcs:ignore
		$schema->baseSalary->value->$type = 'QuantitativeValue'; // phpcs:ignore
		$schema->baseSalary->value->value = $data->salary_amount; // phpcs:ignore

		$schema->baseSalary->value->unitText = $data->salary_payroll; // phpcs:ignore
	}

	if ( ! empty( $data->company ) ) {
		$schema->hiringOrganization        = new stdClass(); // phpcs:ignore
		$schema->hiringOrganization->$type = 'Organization'; // phpcs:ignore
		$schema->hiringOrganization->name  = $data->company; // phpcs:ignore
		if ( ! empty( $data->org_url ) ) {
			$schema->hiringOrganization->sameAs = esc_url( $data->org_url ); // phpcs:ignore
		}
		if ( ! empty( $data->org_logo ) ) {
			$schema->hiringOrganization->logo = esc_url( $data->org_logo ); // phpcs:ignore
		}
	}

	$schema->jobLocation          = new stdClass(); // phpcs:ignore
	$schema->jobLocation->$type   = 'Place'; // phpcs:ignore
	$schema->jobLocation->address = diagnoseo_get_address_schema( $data ); // phpcs:ignore

	$schema->image = get_the_post_thumbnail_url();

	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates SoftwareApplication structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_softwareapplication( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';

	$schema = diagnoseo_create_schema_object( 'SoftwareApplication' );
	if ( ! empty( $data->title ) ) {
		$schema->name = $data->title;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	if ( ! empty( $data->operating_system ) ) {
		$schema->operatingSystem = $data->operating_system; // phpcs:ignore
	}
	if ( ! empty( $data->app_category ) ) {
		$schema->applicationCategory = $data->app_category; // phpcs:ignore
	}
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}

	if ( ! empty( $data->offer_price ) && ! empty( $data->offer_currency ) ) {
		$schema->offers                = new stdClass();
		$schema->offers->$type         = 'Offer';
		$schema->offers->price         = $data->offer_price;
		$schema->offers->priceCurrency = $data->offer_currency;
		$schema->offers->availability  = 'InStock';
	}

	$schema->review           = diagnoseo_get_rating_schema( $data );
	$schema->aggregateRating  = diagnoseo_get_aggregate_rating_schema( $data ); // phpcs:ignore
	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Book structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_book( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';

	$schema = diagnoseo_create_schema_object( 'Book' );

	if ( ! empty( $data->title ) ) {
		$schema->name = $data->title;
	}
	if ( ! empty( $data->url ) ) {
		$schema->url = esc_url( $data->url );
	}
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}

	if ( ! empty( $data->author_name ) ) {
		$schema->author        = new stdClass();
		$schema->author->$type = 'Person';
		$schema->author->name  = $data->author_name;
	}

	$schema->review           = diagnoseo_get_rating_schema( $data );
	$schema->mainEntityOfPage = diagnoseo_get_mainentity_schema( $data ); // phpcs:ignore

	return $schema;
}

/**
 * Generates Event structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_event( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;
	$type = '@type';

	$schema        = diagnoseo_create_schema_object( 'Event' );
	$schema->$type = $data->type;
	if ( ! empty( $data->title ) ) {
		$schema->name = $data->title;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	if ( ! empty( $data->status ) ) {
		$schema->eventStatus = $data->status; // phpcs:ignore
	}
	if ( ! empty( $data->mode ) ) {
		$schema->eventAttendanceMode = $data->mode; // phpcs:ignore
	}
	if ( ! empty( $data->date_start ) ) {
		$schema->startDate = $data->date_start; // phpcs:ignore
	}
	if ( ! empty( $data->date_end ) ) {
		$schema->endDate = $data->date_end; // phpcs:ignore
	}
	$schema->inLanguage = get_locale(); // phpcs:ignore
	if ( has_post_thumbnail() ) {
		$schema->image = get_the_post_thumbnail_url();
	}

	if ( ! empty( $data->venue_name ) ) {
		$schema->location          = new stdClass();
		$schema->location->$type   = 'Place';
		$schema->location->name    = $data->venue_name;
		$schema->location->address = diagnoseo_get_address_schema( $data );
	}

	if ( ! empty( $data->performer_name ) ) {
		$schema->performer        = new stdClass();
		$schema->performer->$type = $data->performer_type;
		$schema->performer->name  = $data->performer_name;
		if ( ! empty( $data->performer_url ) ) {
			$schema->performer->sameAs = esc_url( $data->performer_url );
		}
	}

	if ( ! empty( $data->organizer_name ) ) {
		$schema->organizer        = new stdClass();
		$schema->organizer->$type = $data->organizer_type;
		$schema->organizer->name  = $data->organizer_name;
		if ( ! empty( $data->organizer_url ) ) {
			$schema->organizer->url = esc_url( $data->organizer_url );
		}
	}

	if ( ! empty( $data->offer_url ) || ! empty( $data->offer_price ) ) {
		$schema->offers           = new stdClass();
		$schema->offers->$type    = 'Offer';
		$schema->offers->name     = 'General Admission';
		$schema->offers->category = 'primary';

		if ( ! empty( $data->offer_url ) ) {
			$schema->offers->url = esc_url( $data->offer_url );
		}
		if ( ! empty( $data->offer_price ) ) {
			$schema->offers->price = $data->offer_price;
			if ( ! empty( $data->offer_currency ) ) {
				$schema->offers->priceCurrency = $data->offer_currency;
			}
		}
		$schema->offers->availability = $data->offer_availability;
		if ( ! empty( $data->price_valid_from ) ) {
			$schema->offers->validFrom = $data->price_valid_from;
		}
		if ( ! empty( $data->inv_level ) ) {
			$schema->offers->inventoryLevel = $data->inv_level;
		}
		$schema->offers->availability = $data->offer_availability;
	}

	$schema->review = diagnoseo_get_rating_schema( $data );
	return $schema;
}

/**
 * Generates VideoObject structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_videoobject( $data ) {
	if ( empty( $data ) ) {
		return;
	}
	global $post;

	$schema = diagnoseo_create_schema_object( 'VideoObject' );
	if ( ! empty( $data->title ) ) {
		$schema->name = $data->title;
	}
	if ( ! empty( $data->description ) ) {
		$schema->description = $data->description;
	}
	if ( ! empty( $data->embed_url ) ) {
		$schema->embedUrl = esc_url( $data->embed_url ); // phpcs:ignore
	}
	if ( ! empty( $data->content_url ) ) {
		$schema->contentUrl = esc_url( $data->content_url ); // phpcs:ignore
	}
	if ( ! empty( $data->duration ) ) {
		$schema->duration = $data->duration;
	}
	$schema->uploadDate   = get_the_time( 'c' ); // phpcs:ignore
	$schema->thumbnailUrl = get_the_post_thumbnail_url(); // phpcs:ignore
	$schema->inLanguage   = get_locale(); // phpcs:ignore

	return $schema;
}

/**
 * Generates ImageObject structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_imageobject( $data ) {
	$is_attachment = is_attachment();
	if ( empty( $data ) && ! $is_attachment ) {
		return;
	}

	global $post;
	$attachment_id = $is_attachment ? $post->ID : get_post_thumbnail_id( $post->ID );
	if ( empty( $attachment_id ) ) {
		return;
	}

	$meta        = wp_get_attachment_metadata( $attachment_id );
	$author_info = get_user_by( 'ID', $post->post_author );

	$schema                = diagnoseo_create_schema_object( 'ImageObject' );
	$schema->datePublished = get_the_date( get_option( 'date-format' ) ); // phpcs:ignore
	$schema->width         = $meta['width'];
	$schema->height        = $meta['height'];
	$schema->contentUrl    = wp_get_attachment_url( $attachment_id ); // phpcs:ignore
	$schema->author        = $author_info->display_name;

	if ( ! $is_attachment ) {
		$schema->name            = $data->title;
		$schema->description     = $data->description;
		$schema->contentLocation = $data->location; // phpcs:ignore
	} else {
		$schema->name        = $post->post_title;
		$schema->description = $post->post_content;
	}

	return $schema;
}

/**
 * Generates WebPage structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_webpage( $data ) {
	if ( empty( $data ) ) {
		return;
	}

	global $post;
	$type = '@type';

	$schema              = diagnoseo_create_schema_object( 'WebPage' );
	$schema->name        = get_the_title();
	$schema->description = $data->description;
	$schema->url         = get_the_permalink();
	$schema->image       = get_the_post_thumbnail_url();

	if ( ! empty( $data->publisher_name ) ) {
		$schema->publisher        = new stdClass();
		$schema->publisher->$type = $data->publisher_type;
		$schema->publisher->name  = $data->publisher_name;
	}
	$reviewed_by = diagnoseo_get_reviewedby_schema( $data );
	if ( ! empty( $reviewed_by ) ) {
		$schema->reviewedBy = $reviewed_by; // phpcs:ignore
	}

	return $schema;
}

/**
 * Generates FAQPage structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_faqpage( $data ) {
	if ( empty( $data ) ) {
		return;
	}

	global $post;
	$type = '@type';

	$schema              = diagnoseo_create_schema_object( 'FAQPage' );
	$schema->name        = get_the_title();
	$schema->description = $data->description;
	$schema->url         = get_the_permalink();
	$schema->image       = get_the_post_thumbnail_url();
	$schema->mainEntity  = new stdClass(); // phpcs:ignore

	$schema->mainEntity->$type = 'Question'; // phpcs:ignore
	$schema->mainEntity->name  = $data->main_question; // phpcs:ignore

	$schema->mainEntity->acceptedAnswer        = new stdClass(); // phpcs:ignore
	$schema->mainEntity->acceptedAnswer->$type = 'Answer'; // phpcs:ignore
	$schema->mainEntity->acceptedAnswer->text  = $data->main_answer; // phpcs:ignore

	if ( ! empty( $data->publisher_name ) ) {
		$schema->publisher        = new stdClass();
		$schema->publisher->$type = $data->publisher_type;
		$schema->publisher->name  = $data->publisher_name;
	}

	$reviewed_by = diagnoseo_get_reviewedby_schema( $data );
	if ( ! empty( $reviewed_by ) ) {
		$schema->reviewedBy = $reviewed_by; // phpcs:ignore
	}

	return $schema;
}

/**
 * Generates AboutPage structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_aboutpage( $data ) {
	if ( empty( $data ) ) {
		return;
	}

	global $post;
	$type = '@type';

	$schema              = diagnoseo_create_schema_object( 'AboutPage' );
	$schema->name        = get_the_title();
	$schema->description = $data->description;
	$schema->url         = get_the_permalink();
	$schema->image       = get_the_post_thumbnail_url();

	if ( ! empty( $data->publisher_name ) ) {
		$schema->publisher        = new stdClass();
		$schema->publisher->$type = $data->publisher_type;
		$schema->publisher->name  = $data->publisher_name;
	}

	$reviewed_by = diagnoseo_get_reviewedby_schema( $data );
	if ( ! empty( $reviewed_by ) ) {
		$schema->reviewedBy = $reviewed_by; // phpcs:ignore
	}

	return $schema;
}

/**
 * Generates ContactPage structured data
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_schema_contactpage( $data ) {
	if ( empty( $data ) ) {
		return;
	}

	global $post;
	$type = '@type';

	$schema              = diagnoseo_create_schema_object( 'ContactPage' );
	$schema->name        = get_the_title();
	$schema->description = $data->description;
	$schema->url         = get_the_permalink();
	$schema->image       = get_the_post_thumbnail_url();

	if ( ! empty( $data->publisher_name ) ) {
		$schema->publisher        = new stdClass();
		$schema->publisher->$type = $data->publisher_type;
		$schema->publisher->name  = $data->publisher_name;
	}

	return $schema;
}

/**
 * Creates the schema object with basic fields
 *
 * @param String $type Schema object type.
 */
function diagnoseo_create_schema_object( $type ) {
	$context             = '@context';
	$type_field          = '@type';
	$schema              = new stdClass();
	$schema->$context    = 'https://schema.org';
	$schema->$type_field = $type;
	return $schema;
}

/**
 * Returns the author schema for current post
 */
function diagnoseo_get_author_schema() {
	global $post;

	$author              = new stdClass();
	$id_field            = '@id';
	$type_field          = '@type';
	$author->$type_field = 'Person';
	$author_id           = $post->post_author;
	$author_info         = get_user_by( 'ID', $author_id );
	$author->$id_field   = get_author_posts_url( $author_id );
	$author->url         = get_author_posts_url( $author_id );
	$author->name        = $author_info->display_name;

	return $author;
}

/**
 * Builds the rating schema
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_get_rating_schema( $data ) {
	$type = '@type';
	$id   = '@id';
	global $post;

	$schema                = new stdClass();
	$schema->$type         = 'Review';
	$schema->datePublished = get_the_time( 'c' ); // phpcs:ignore
	$schema->dateModified  = get_the_modified_time( 'c' ); // phpcs:ignore

	if ( ! empty( $data->title ) ) {
		$schema->itemReviewed = $data->title; // phpcs:ignore
	} elseif ( ! empty( $data->name ) ) {
		$schema->itemReviewed = $data->name; // phpcs:ignore
	} elseif ( ! empty( $data->headline ) ) {
		$schema->itemReviewed = $data->headline; // phpcs:ignore
	} else {
		$schema->itemReviewed = get_the_title(); // phpcs:ignore
	}

	$schema->author = new stdClass();
	if ( ! empty( $data->publisher_type ) && ! empty( $data->publisher_name ) ) {
		$schema->author->$type = $data->publisher_type;
		$schema->author->name  = $data->publisher_name;
	} else {
		$schema->author->$id   = get_author_posts_url( $post->post_author );
		$schema->author->$type = 'Person';
		$author_info           = get_user_by( 'ID', $post->post_author );
		$schema->author->url   = get_author_posts_url( $post->post_author );
		$schema->author->name  = $author_info->display_name;
	}

	if ( ! empty( $data->rating ) ) {
		$schema->reviewRating              = new stdClass(); // phpcs:ignore
		$schema->reviewRating->$type       = 'Rating'; // phpcs:ignore
		$schema->reviewRating->ratingValue = $data->rating; // phpcs:ignore

		if ( ! empty( $data->rating_min ) || 0 === $data->rating_min ) {
			$schema->reviewRating->worstRating = $data->rating_min; // phpcs:ignore
		}
		if ( ! empty( $data->rating_max ) ) {
			$schema->reviewRating->bestRating = $data->rating_max; // phpcs:ignore
		}
		if ( ! empty( $data->rating_count ) ) {
			$schema->reviewRating->ratingCount = $data->rating_count; // phpcs:ignore
		}
	}

	return $schema;
}

/**
 * Builds aggregate rating object
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_get_aggregate_rating_schema( $data ) {
	$type = '@type';

	$schema              = new stdClass();
	$schema->$type       = 'AggregateRating';
	$schema->ratingValue = $data->rating; // phpcs:ignore

	if ( ! empty( $data->rating_count ) ) {
		$schema->ratingCount = $data->rating_count; // phpcs:ignore
	}
	if ( ! empty( $data->rating ) ) {
		$schema->ratingValue = $data->rating; // phpcs:ignore
	}

	return $schema;
}
/**
 * Builds address schema
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_get_address_schema( $data ) {
	$type = '@type';
	$id   = '@id';

	$schema        = new stdClass();
	$schema->$type = 'PostalAddress';

	if ( ! empty( $data->street_address ) ) {
		$schema->streetAddress = $data->street_address; // phpcs:ignore
	}
	if ( ! empty( $data->locality ) ) {
		$schema->addressLocality = $data->locality; // phpcs:ignore
	}
	if ( ! empty( $data->region ) ) {
		$schema->addressRegion = $data->region; // phpcs:ignore
	}
	if ( ! empty( $data->postal_code ) ) {
		$schema->postalCode = $data->postal_code; // phpcs:ignore
	}
	if ( ! empty( $data->country ) ) {
		$schema->addressCountry = $data->country; // phpcs:ignore
	}

	return $schema;
}
/**
 * Builds ReviewedBy schema
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_get_reviewedby_schema( $data ) {
	if ( empty( $data->reviewedby_name ) && empty( get_option( 'diagnoseo_reviewedby_name', '' ) ) ) {
		return '';
	}
	$type = '@type';
	$id   = '@id';

	$schema        = new stdClass();
	$schema->$type = empty( $data->reviewedby_type ) ? get_option( 'diagnoseo_reviewedby_type', 'Person' ) : $data->reviewedby_type;
	$schema->name  = empty( $data->reviewedby_name ) ? get_option( 'diagnoseo_reviewedby_name', '' ) : $data->reviewedby_name;
	$schema->url   = empty( $data->reviewedby_url ) ? get_option( 'diagnoseo_reviewedby_url', '' ) : $data->reviewedby_url;

	return $schema;
}

/**
 * Builds MainEntityOfPage schema
 *
 * @param Object $data Information to be included in the structured data object.
 */
function diagnoseo_get_mainentity_schema( $data ) {
	$type = '@type';
	$id   = '@id';
	global $post;

	$schema             = new stdClass();
	$schema->$type      = 'WebPage';
	$schema->$id        = get_the_permalink( $post->ID );
	$schema->name       = get_the_title( $post->ID );
	$schema->reviewedBy = diagnoseo_get_reviewedby_schema( $data ); // phpcs:ignore

	return $schema;
}


/**
 * Displays Person/Organization schema based on options
 */
function diagnoseo_site_owner_schema() {
	include DIAGNOSEO_INCLUDES_PATH . 'option-defaults.php';

	$enabled = get_option( 'diagnoseo_schema_owner', true );
	$type    = get_option( 'diagnoseo_site_owner_type', $diagnoseo_defaults['diagnoseo_site_owner_type'] );
	if ( ! $enabled || empty( $type ) ) {
		return;
	}

	$schema            = diagnoseo_create_schema_object( $type );
	$schema->name      = get_option( 'diagnoseo_site_owner_name' );
	$schema->url       = get_option( 'diagnoseo_site_owner_url' );
	$schema->email     = get_option( 'diagnoseo_site_owner_email' );
	$schema->telephone = get_option( 'diagnoseo_site_owner_phone' );
	$schema->image     = get_option( 'diagnoseo_site_owner_image' );
	$address           = get_option( 'diagnoseo_site_owner_street_address' );

	if ( ! empty( $address ) ) {
		$schema->address                  = diagnoseo_create_schema_object( 'PostalAddress' );
		$schema->address->streetAddress   = $address;
		$schema->address->addressLocality = get_option( 'diagnoseo_site_owner_locality' );
		$schema->address->postalCode      = get_option( 'diagnoseo_site_owner_postal_code' );
		$schema->address->addressRegion   = get_option( 'diagnoseo_site_owner_region' );
		$schema->address->addressCountry  = get_option( 'diagnoseo_site_owner_country' );
	}

	$link_settings = array( 'Facebook', 'Twitter', 'Instagram', 'LinkedIn', 'Pinterest', 'YouTube', 'Wikipedia' );
	$social_links  = array();

	foreach ( $link_settings as $option ) {
		$option_name = 'diagnoseo_social_' . strtolower( $option );
		$link        = get_option( $option_name );
		if ( ! empty( $link ) ) {
			$social_links[] = esc_url( $link );
		}
	}

	$schema->sameAs = $social_links; // phpcs:ignore

	?>
<!-- DiagnoSEO Site Owner Structured Data -->
<script type="application/ld+json"><?php echo wp_json_encode( $schema, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE ); ?></script>
<!-- DiagnoSEO Site Owner Structured Data End -->
	<?php
}
add_action( 'wp_head', 'diagnoseo_site_owner_schema' );
